package com.jiuqi.util.zip;

import java.util.zip.ZipException;
import java.util.zip.CRC32;

public class AsiExtraField implements ZipExtraField, UnixStat, Cloneable
{
    private static final ZipShort HEADER_ID;
    private int mode;
    private int uid;
    private int gid;
    private String link;
    private boolean dirFlag;
    private CRC32 crc;
    
    static {
        HEADER_ID = new ZipShort(30062);
    }
    
    public AsiExtraField() {
        this.mode = 0;
        this.uid = 0;
        this.gid = 0;
        this.link = "";
        this.dirFlag = false;
        this.crc = new CRC32();
    }
    
    public ZipShort getHeaderId() {
        return AsiExtraField.HEADER_ID;
    }
    
    public ZipShort getLocalFileDataLength() {
        return new ZipShort(14 + this.getLinkedFile().getBytes().length);
    }
    
    public ZipShort getCentralDirectoryLength() {
        return this.getLocalFileDataLength();
    }
    
    public byte[] getLocalFileDataData() {
        final byte[] data = new byte[this.getLocalFileDataLength().getValue() - 4];
        System.arraycopy(new ZipShort(this.getMode()).getBytes(), 0, data, 0, 2);
        final byte[] linkArray = this.getLinkedFile().getBytes();
        System.arraycopy(new ZipLong(linkArray.length).getBytes(), 0, data, 2, 4);
        System.arraycopy(new ZipShort(this.getUserId()).getBytes(), 0, data, 6, 2);
        System.arraycopy(new ZipShort(this.getGroupId()).getBytes(), 0, data, 8, 2);
        System.arraycopy(linkArray, 0, data, 10, linkArray.length);
        this.crc.reset();
        this.crc.update(data);
        final long checksum = this.crc.getValue();
        final byte[] result = new byte[data.length + 4];
        System.arraycopy(new ZipLong(checksum).getBytes(), 0, result, 0, 4);
        System.arraycopy(data, 0, result, 4, data.length);
        return result;
    }
    
    public byte[] getCentralDirectoryData() {
        return this.getLocalFileDataData();
    }
    
    public void setUserId(final int uid) {
        this.uid = uid;
    }
    
    public int getUserId() {
        return this.uid;
    }
    
    public void setGroupId(final int gid) {
        this.gid = gid;
    }
    
    public int getGroupId() {
        return this.gid;
    }
    
    public void setLinkedFile(final String name) {
        this.link = name;
        this.mode = this.getMode(this.mode);
    }
    
    public String getLinkedFile() {
        return this.link;
    }
    
    public boolean isLink() {
        return this.getLinkedFile().length() != 0;
    }
    
    public void setMode(final int mode) {
        this.mode = this.getMode(mode);
    }
    
    public int getMode() {
        return this.mode;
    }
    
    public void setDirectory(final boolean dirFlag) {
        this.dirFlag = dirFlag;
        this.mode = this.getMode(this.mode);
    }
    
    public boolean isDirectory() {
        return this.dirFlag && !this.isLink();
    }
    
    public void parseFromLocalFileData(final byte[] data, final int offset, final int length) throws ZipException {
        final long givenChecksum = new ZipLong(data, offset).getValue();
        final byte[] tmp = new byte[length - 4];
        System.arraycopy(data, offset + 4, tmp, 0, length - 4);
        this.crc.reset();
        this.crc.update(tmp);
        final long realChecksum = this.crc.getValue();
        if (givenChecksum != realChecksum) {
            throw new ZipException("bad CRC checksum " + Long.toHexString(givenChecksum) + " instead of " + Long.toHexString(realChecksum));
        }
        final int newMode = new ZipShort(tmp, 0).getValue();
        final byte[] linkArray = new byte[(int)new ZipLong(tmp, 2).getValue()];
        this.uid = new ZipShort(tmp, 6).getValue();
        this.gid = new ZipShort(tmp, 8).getValue();
        if (linkArray.length == 0) {
            this.link = "";
        }
        else {
            System.arraycopy(tmp, 10, linkArray, 0, linkArray.length);
            this.link = new String(linkArray);
        }
        this.setDirectory((newMode & 0x4000) != 0x0);
        this.setMode(newMode);
    }
    
    protected int getMode(final int mode) {
        int type = 32768;
        if (this.isLink()) {
            type = 40960;
        }
        else if (this.isDirectory()) {
            type = 16384;
        }
        return type | (mode & 0xFFF);
    }
}
